//
//  sttyl.c 
//
//      AUTHOR: Russell Lowke
//   CSCIE-215: Assignment Three
//
// **** NEED TO FIX COMMENTS
//
//       Usage: pfind [startDir] [fileName]
//              pfind [-d startDir] [-f fileName]
//
//        Args:  startDir   // the name of the directory being searched through
//               fileName   // the name of the file being searched for
//
// Description: mimics the basic functionality of the the Unix find
//              command, searching through the directory [startDir], and all
//              directories therein for files or directories named [fileName].  
//              When found the full path of the file or directory is
//              printed using printf.
//
//      Method: main reads args, passing parameters to recursive searchdir.
//              searchdir opens directory, searching for and printing any match
//              When a directory is found searchdir calls itself using that directory
//              as the startDir.
//
//       Notes: symbolic links are not followed (see -follow in Unix find),
//              mainly as it may result in infinite loops, lstat being
//              used instead of stat.
//
//
//


// include header
#include                "sttyl.h"

#define INPUTFLAG    1
#define OUTPUTFLAG   2
#define CONTROLFLAG  3
#define LOCALFLAG    4

//
// Table of attributes

// TABDLY ???
struct flagTable activeFlags[] = {
    {"icrnl" , ICRNL , INPUTFLAG  , "map CR to NL (ala CRMOD)"             },
    {"onlcr" , ONLCR , OUTPUTFLAG , "map NL to CR-NL (ala CRMOD)"          },
    {"echo"  , ECHO  , LOCALFLAG  , "enable echoing"                       },
    {"echoe" , ECHOE , LOCALFLAG  , "visually erase chars"                 },
    {"olcuc" , OLCUC , OUTPUTFLAG , "map lowercase to uppercase on output" },
    {"tabs"  , TAB3  , LOCALFLAG  , "expand tabs on output"                },
    {"icanon", ICANON, LOCALFLAG  , "canonicalize input lines"             },
    {"isig"  , ISIG  , LOCALFLAG  , "enable signals INTR, QUIT, [D]SUSP"   },
    {"hupcl" , HUPCL , CONTROLFLAG, "hang up on last close"                },
    {"cread" , CREAD , CONTROLFLAG, "enable receiver"                      },
    {"brkint", BRKINT, INPUTFLAG  , "map BREAK to SIGINTR"                 },
    {"inpck" , INPCK , INPUTFLAG  , "disable checking of parity errors"    },
    {"iexten", IEXTEN, LOCALFLAG  , "enable FLUSHO and LNEXT"              },
    { NULL   , 0     , 0          ,  NULL                                  }};

struct keyTable activeKeys[] = {
    {"intr"  , VINTR },
    {"erase" , VERASE},
    {"kill"  , VKILL },
    {"start" , VSTART},
    {"stop"  , VSTOP },
    { NULL   , 0     }};


int main(int ac, char **av)
{
    struct termios info;    // main struct to hold terminal info
    int    i,               // general purpose counters
           myIndex,         // index of Flag or Key item
           switchIs;        // indicates if flag is turning ON or OFF
    char   myAscii;         // for holding ascii value to set arg flag to

    // load terminal attributes into info
    if (tcgetattr(0, &info) < 0)
        oops("tcgetattr error");
    
    // loop through arguments passed into main via ac and av
    // Start at av[1] as av[0] is the program name, in this case "sttyl"
    for ( i = 1; i < ac; i++ ) {
        
        // ensure myAscii is empty
        myAscii = 0;
        
        // test for key attribute
        if ( (myIndex = isKeyName(av[i])) ) {
            
            // ensure next arg is there?
            if ( i+1 < ac) {
                i++;      // increment to next arg
                
                // check length of arg
                if (strlen(av[i]) == 1)                                 
                    myAscii = av[i][0];
                else if ( strlen(av[i]) == 2 && av[i][0] == '^' )
                    myAscii = asciiCharToCtrl(av[i][1]);                // ascii is char 1.    
            }
            
            // set char key or fatal
            if (myAscii)
                info.c_cc[activeKeys[myIndex].theValue] = myAscii;
            else
                fatal("Invalid argument");
        }
            
        // switchIs to 0, or 1 if '-' operator
        switchIs = 0;
        if (av[i][0] == '-')
            switchIs = 1;

        // is arg a flag attribute?
        if ( (myIndex = isFlagName(&av[i][switchIs])) )
            setFlag(&info, myIndex, switchIs);                   

        // ERROR, can't deal with arg
    }
    
    tcsetattr(0, TCSANOW, &info);

    displaySettings(&info);
    
    return (0);
}


//
// set a flag in the info struct
//
void setFlag(struct termios *info, int myIndex, int switchIs)
{
    if (switchIs) {
        
        // Switch OFF
        switch (activeFlags[myIndex].flagType) {
            case INPUTFLAG:
                info->c_iflag &= ~activeFlags[myIndex].theValue;
                break;
            case OUTPUTFLAG:
                info->c_oflag &= ~activeFlags[myIndex].theValue;
                break;
            case CONTROLFLAG:
                info->c_cflag &= ~activeFlags[myIndex].theValue;
                break;
            case LOCALFLAG:
                info->c_lflag &= ~activeFlags[myIndex].theValue;
                break;
        }
    }
    else {
        
        // Switch ON
        switch (activeFlags[myIndex].flagType) {
            case INPUTFLAG:
                info->c_iflag |= activeFlags[myIndex].theValue;
                break;
            case OUTPUTFLAG:
                info->c_oflag |= activeFlags[myIndex].theValue;
                break;
            case CONTROLFLAG:
                info->c_cflag |= activeFlags[myIndex].theValue;
                break;
            case LOCALFLAG:
                info->c_lflag |= activeFlags[myIndex].theValue;
                break;
            }
    }
}

//
//  convert standard ascii chars to control char ascii
// 
int asciiCharToCtrl(char myChar)
{                                                       
    // convert "^?" to backspace
    if (myChar == '?')
        return 127;
                                
    // convert letter ascii to cntrl ascii
    if (myChar > 96) 
        myChar = myChar - 'a' + 1; // lower case
    else
        myChar = myChar - 'A' + 1; // upper case

    if (myChar < 0 || myChar > 31)
        return 0;                        // BAD VALUE

    return myChar;
}


int isFlagName(char *myString)
{
    int i;
    
    for ( i = 0; activeFlags[i].theValue ; i++ ) {
        if ( strcmp(activeFlags[i].attribute, myString) == 0 )
            return i;
    }

    return 0;
}

int isKeyName(char *myString)
{
    int i;
    
    for ( i = 0; activeKeys[i].theValue ; i++ ) {
        if ( strcmp(activeKeys[i].attribute, myString) == 0 )
            return i;
    }

    return 0;
}

void displaySettings(struct termios *info)
{    
    int i;  
        
    showbaud( cfgetospeed( info ));  // get and show baud rate
    printf("\n");

    for ( i = 0; activeKeys[i].theValue ; i++ ) {
        printf("%s = ", activeKeys[i].attribute);
        printASCII(info->c_cc[activeKeys[i].theValue]);
    }
    printf("\n");
   
    // test for flag attribute
    for ( i = 0; activeFlags[i].theValue ; i++ ) {
        printFlag( info, activeFlags[i].attribute,
                          activeFlags[i].theValue,
                          activeFlags[i].flagType);
    }
    printf("\n"); 
}


//
//
void printFlag( struct termios *ttyp,
                char *flagName,
                int theFlag, 
                int flagType)
{
    int theValue = 0;

    switch (flagType) {
        case INPUTFLAG:
            theValue = ttyp->c_iflag;
            break;
        case OUTPUTFLAG:
            theValue = ttyp->c_oflag;
            break;
        case CONTROLFLAG:
            theValue = ttyp->c_cflag;
            break;
        case LOCALFLAG:
            theValue = ttyp->c_lflag;
            break;
    }


    if (theValue & theFlag)
            printf("%s ", flagName);
    else
            printf("-%s ", flagName);
}

//
//  Given an ASCII code print the readible symbol
//
void printASCII(int myAscii)
{
    if (myAscii == 127)
        printf("^?; ");
    else if (myAscii > 0 || myAscii < 32)
        printf("^%c; ", myAscii -1 + 'A');
    else
        printf("%c; ", myAscii);    
}


//
//      prints the speed in english
//
void showbaud( int thespeed )
{
        printf("speed ");
        switch ( thespeed ){
                case B300:      printf("300");        break;
                case B600:      printf("600");        break;
                case B1200:     printf("1200");       break;
                case B1800:     printf("1800");       break;
                case B2400:     printf("2400");       break;
                case B4800:     printf("4800");       break;
                case B9600:     printf("9600");       break;
                default:        printf("Fast");       break;
        }
        printf(" baud; ");
}

//
// Input flags - software input processing
//
struct flaginfo inputFlags[] = {
                  {IGNBRK    , "ignore BREAK condition"},
                  {BRKINT    , "map BREAK to SIGINTR"},
                  {IGNPAR    , "ignore (discard) parity errors"},
                  {PARMRK    , "mark parity and framing errors"},
                  {INPCK     , "disable checking of parity errors"},
                  {ISTRIP    , "strip 8th bit off chars"},
                  {INLCR     , "map NL into CR"},
                  {IGNCR     , "ignore CR"},
                  {ICRNL     , "map CR to NL (ala CRMOD)"},
                  {IXON      , "enable output flow control"},
                  {IXOFF     , "enable input flow control"},
                  {IXANY     , "any char will restart after stop"},
                  {IUCLC     , "DUMMY VALUE Map upper to lower case on input"},
                  {IFLOW     , "enable output flow control"},
                  {ITANDEM   , "enable input flow control"},
                  {IMAXBEL   , "ring bell on input queue full"},
                  {0         ,  NULL }};
//
// Output flags - software output processing
//
struct flaginfo outputFlags[] = {
                     {OPOST     , "enable following output processing"},
                     {ONLCR     , "map NL to CR-NL (ala CRMOD)"},
                     {OLCUC     , "Map lower case to upper on output"},
                     {OCRNL     , "Map CR to NL on output"},
                     {ONOCR     , "No CR output at column 0"},
                     {ONLRET    , "NL performs CR function"},
                     {OFILL     , "Use fill characters for delay"},
                     {OFDEL     , "fill is DEL, else NUL"},
                     {NLDLY     , "\\n delay"},
                     {NL0       , "NL0"},
                     {NL1       , "NL1 tty 37"},
                     {NL2       , "NL2 vt05"},
                     {NL3       , "NL3"},
                     {TABDLY    , "horizontal tab delay"},
                     {TAB0      , "TAB0"},
                     {TAB1      , "TAB1 tty 37"},
                     {TAB2      , "TAB2"},
                     {TAB3      , "TAB3 expand tabs on output"},
                     {CRDLY     , "\\r delay"},
                     {CR0       , "CR0"},
                     {CR1       , "CR1 tn 300"},
                     {CR2       , "CR2 tty 37"},
                     {CR3       , "CR3 concept 100"},
                     {FFDLY     , "Form feed delay"},
                     {FF0       , "FF0"},
                     {FF1       , "FF1"},
                     {BSDLY     , "\\b delay"},
                     {BS0       , "BS0"},
                     {BS1       , "BS1"},
                     {VTDLY     , "vertical tab delay"},
                     {VT0       , "VT0"},
                     {VT1       , "VT1 tty 37"},
                     {ONLCRNL   , "ONLCRNL"},
                     {OXTABS    , "expand tabs to spaces"},
                     {ONOEOT    , "discard EOT's (^D) on output)"},
                     {0         ,  NULL }};                  
//
// Control flags - hardware control of terminal
//
struct flaginfo cntrFlags[] = {
                     {CSIZE     , "character size mask"},
                     {CSTOPB    , "send 2 stop bits"},
                     {CREAD     , "enable receiver"},
                     {PARENB    , "parity enable"},
                     {PARODD    , "odd parity, else even"},
                     {HUPCL     , "hang up on last close"},
                     {CLOCAL    , "ignore modem status lines"},
                     {CRTSCTS   , "RTS/CTS flow control"},
                     {0         ,  NULL }};

                     //{CS5       , "5 bits (pseudo)"},
                     //{CS6       , "6 bits"},
                     //{CS7       , "7 bits"},
                     //{CS8       , "8 bits"},

// 
// "Local" flags - dumping ground for other state
//
// Warning: some flags in this structure begin with
// the letter "I" and look like they belong in the
// input flag.
//
struct flaginfo localFlags[] = {
                     {ECHOE     , "visually erase chars"},
                     {ECHOK     , "echo NL after line kill"},
                     {ECHO      , "enable echoing"},
                     {ECHONL    , "echo NL even if ECHO is off"},
                     {ISIG      , "enable signals INTR, QUIT, [D]SUSP"},
                     {ICANON    , "canonicalize input lines"},
                     {IEXTEN    , "enable FLUSHO and LNEXT"},
                     {NOFLSH    , "don't flush after interrupt"},
                     {TOSTOP    , "stop background jobs from output"},
                     {XCASE     , "Cononical upper/lower presentation"},
                     {ECHOKE    , "visual erase for line kill"},
                     {ECHOPRT   , "visual erase mode for hardcopy"},
                     {ECHOCTL   , "echo control chars as ^(Char)"},
                     {ALTWERASE , "use alternate WERASE algorithm"},
                     {MDMBUF    , "flow control output via Carrier"},
                     {FLUSHO    , "output being flushed (state)"},
                     {NOHANG    , "this should go away"},
                     {PENDIN    , "retype pending input (state)"},
                     {NOKERNINFO, "Disable printing kernel info"},
                     {0         , NULL }};




//
//      show the values of two of the flag sets_: c_iflag and c_lflag
//      adding c_oflag and c_cflag is pretty routine - just add new
//      tables above and a bit more code below.
//                   
void showSomeFlags( struct termios *ttyp )
{
        printf("input flags\n");
        showFlagSet( ttyp->c_iflag, inputFlags  );
        printf("output flags\n");
        showFlagSet( ttyp->c_oflag, outputFlags );
        printf("cntrl flags\n");
        showFlagSet( ttyp->c_cflag, cntrFlags   );
        printf("local flags\n");
        showFlagSet( ttyp->c_lflag, localFlags  );
}

void showFlagSet( int thevalue, struct flaginfo thebitnames[] )
//
// check each bit pattern and display descriptive title
//
{
        int i;
        
        for ( i = 0; thebitnames[i].fl_value ; i++ ) {
                printf( "  %s is ", thebitnames[i].fl_name);
                if ( thevalue & thebitnames[i].fl_value )
                        printf("ON\n");
                else
                        printf("OFF\n");
        }
}
